from vaccel.session import Session
from typing import List
from vaccel.genop import Genop, VaccelArg, VaccelOpType
import struct
import copy

class Pynq_vector_add:
    """A Pynq vector add model vAccel resource.
    
    Attributes:
        __op__: The genop operation type
        def_arg_write (bytes): The result of the operation
    """

    __op__ = VaccelOpType.VACCEL_PYNQ_VECTOR_ADD
    
    def_arg_write: bytes = bytes(100 * " ", encoding="utf-8")

    @staticmethod
    def __genop__(arg_read: List[VaccelArg], arg_write: List[VaccelArg], index: int) -> int:
        """Performs the genop operation provided in arg_read.

        Args:
            arg_read: `list` 
            arg_write: `list`
            index: `int`

        Returns:
        The content of the arg_write indicated by index.
        """
        ses = Session(flags=0)
        Genop.genop(ses, arg_read, arg_write)
        return struct.unpack('d', arg_write[index].raw_content[:8])[0]

    @classmethod
    def pynq_vector_add(cls, len_a:int, len_b:int):
        """Executes Pynq vector add operation using vAccel over genop.

        Args:
            len_a: An integer giving the length of the array a
            len_b: An integer giving the length of the array b

        Returns:
            a: A float for the array a
            b: A float for the array b
            c: A float for the result of the addition
        """
        
        arg_read = [VaccelArg(data=int(cls.__op__)),
                    VaccelArg(data=len_a),VaccelArg(data=len_b)]
        arg_write = [VaccelArg(data=cls.def_arg_write)]
        return cls.__genop__(arg_read=arg_read, arg_write=arg_write, index=0)